#ifndef evalxH
#define evalxH
#include "fltpt.h"

#define XVARLEN          20              // Max length of variable names.
#define MAXXVARS         50              // Max user-defined variables.
#define XTOKLEN          80              // Max token length.

enum {XVAR=1,XDEL,XNUM};

typedef struct
{
   char name[XVARLEN + 1];              // Variable name.
   bcd32 value;                         // Variable value.
} XVARIABLE;

typedef struct
{
   char *name;                          // Function name.
   int   args;                          // Number of arguments to expect.
   void  (*func1)(bcd32 ,bcd32);        // Pointer to function.
   void  (*func2)(bcd32,bcd32,bcd32);
} XFUNCTION;

#define iswhite(c)  (c == ' ' || c == '\t')
#define isnumer(c)  ((c >= '0' && c <= '9') || c == '.')
#define isexpnum(c) ((c >= '0' && c <= '9'))

// Codes returned from the evaluator.
#define E_OK           0        // Successful evaluation.
#define E_SYNTAX       1        // Syntax error.
#define E_UNBALAN      2        // Unbalanced parenthesis.
#define E_DIVZERO      3        // Attempted division by zero.
#define E_UNKNOWN      4        // Reference to unknown variable.
#define E_MAXVARS      5        // Maximum variables exceeded.
#define E_BADFUNC      6        // Unrecognised function.
#define E_NUMARGS      7        // Wrong number of arguments to function.
#define E_NOARG        8        // Missing an argument to a function.
#define E_EMPTY        9        // Empty expression.

static char *ErrMsgs[] = {"OK",
                "Syntax error",
                "Unbalanced parenthesis",
                "Divide by zero",
                "Unknown variable",
                "Maximum variables exceeded",
                "Unrecognized function",
                "Wrong number of arguments",
                "Missing argument",
                "Empty expression"
                };
int GetNumArgs(int idx);
char *GetXFuncs(int idx);
char *GetXConst(int idx);
int SetXValue(char *name,bcd32 val);

int EvaluateX(char *e,bcd32 res,int *epos);

#endif
