//  xfloat.h -- class for extended precision bcd arithmetic
//  (C) 2000, C. Bond. All rights reserved.
//

#ifndef xfloatH
#define xfloatH

#include <iostream.h>
#include "fltpt.h"
#include "fpconv.h"

class xfloat {
// Implementation
    bcd32 xbcd;
    public:
// Constructors
    xfloat(bcd32=k0);
    xfloat(char *s);
    xfloat(double);
// Unary operators
    xfloat operator-();
    xfloat operator+();
// Binary operators
    xfloat operator+(xfloat op2);
    xfloat operator+(double z);
    xfloat operator+(bcd32 op2);
    friend xfloat operator+(double z,xfloat op2);
    friend xfloat operator+(bcd32 op1,xfloat op2);

    xfloat operator-(xfloat op2);
    xfloat operator-(double z);
    xfloat operator-(bcd32 op2);
    friend xfloat operator-(double z,xfloat op2);
    friend xfloat operator-(bcd32 op1,xfloat op2);

    xfloat operator*(xfloat op2);
    xfloat operator*(double z);
    xfloat operator*(bcd32 op2);
    friend xfloat operator*(double z,xfloat op2);
    friend xfloat operator*(bcd32 op1,xfloat op2);

    xfloat operator/(xfloat op2);
    xfloat operator/(double z);
    xfloat operator/(bcd32 op2);
    friend xfloat operator/(double z,xfloat op2);
    friend xfloat operator/(bcd32 op1,xfloat op2);

    xfloat operator=(bcd32 op2);
    xfloat operator=(xfloat op2);
    xfloat operator=(double z);
    xfloat operator=(char *str);
    xfloat operator+=(xfloat op2);
    xfloat operator+=(double z);
    xfloat operator-=(xfloat op2);
    xfloat operator-=(double z);
    xfloat operator*=(xfloat op2);
    xfloat operator*=(double z);
    xfloat operator/=(xfloat op2);
    xfloat operator/=(double z);
// Relational operators
    bool operator>(xfloat op2);
    bool operator>(double z);
    bool operator<(xfloat op2);
    bool operator<(double z);
    bool operator==(xfloat op2);
    bool operator==(double z);
    bool operator>=(xfloat op2);
    bool operator>=(double z);
    bool operator<=(xfloat op2);
    bool operator<=(double z);
    bool operator!=(xfloat op2);
    bool operator!=(double z);
    bool operator!();
// Type coercion
    operator int () { return (int)cb22dbl(xbcd); };
    operator double () { return cb22dbl(xbcd); };
// I/O operators
    friend ostream &operator<<(ostream &stream,xfloat a);
    friend istream &operator>>(istream &stream,xfloat &a);
// Math Functions
    friend xfloat max(xfloat a,xfloat b);
    friend xfloat min(xfloat a,xfloat b);
    friend xfloat abs(xfloat a);
    friend xfloat sqrt(xfloat a);
    friend xfloat exp(xfloat a);
    friend xfloat log(xfloat a);
    friend xfloat pow(xfloat a,xfloat b);
    friend xfloat pow(xfloat a,double z);
    friend xfloat pow(double z,xfloat a);
    friend xfloat log10(xfloat a);
    friend xfloat alog10(xfloat a);
    friend xfloat sin(xfloat a);
    friend xfloat cos(xfloat a);
    friend xfloat tan(xfloat a);
    friend xfloat cot(xfloat a);
    friend xfloat csc(xfloat a);
    friend xfloat sec(xfloat a);
    friend xfloat asin(xfloat a);
    friend xfloat acos(xfloat a);
    friend xfloat atan(xfloat a);
    friend xfloat atan2(xfloat x,xfloat y);
    friend xfloat acsc(xfloat a);
    friend xfloat asec(xfloat a);
    friend xfloat acot(xfloat a);
    friend xfloat sinh(xfloat a);
    friend xfloat cosh(xfloat a);
    friend xfloat tanh(xfloat a);
    friend xfloat coth(xfloat a);
    friend xfloat sech(xfloat a);
    friend xfloat csch(xfloat a);
    friend xfloat asinh(xfloat a);
    friend xfloat acosh(xfloat a);
    friend xfloat atanh(xfloat a);
    friend xfloat acoth(xfloat a);
    friend xfloat asech(xfloat a);
    friend xfloat acsch(xfloat a);
    friend xfloat gamma(xfloat a);
    friend xfloat factorial(int n);
    friend xfloat round(xfloat a,int n);
    friend xfloat intgr(xfloat a);
    friend xfloat frac(xfloat a);
    friend xfloat hypot(xfloat a,xfloat b);
    friend xfloat poly(bcd32 coeff,xfloat x,int n);
    friend xfloat poly(xfloat *a,xfloat x,int n);
// Misc Functions
    void show();
    void get();
    friend void x2str(xfloat a,char *str);
    friend double x2dbl(xfloat a);
    friend void x2bcd32(xfloat a,bcd32 b);
};
#endif
